<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2018 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
 * API2 class for orders (admin)
 *
 * @category   Mage
 * @package    Mage_Sales
 * @author     Magento Core Team <core@magentocommerce.com>
 */
class Nostress_Koongoorderapi_Model_Api2_Order extends Mage_Api2_Model_Resource
{
    /**
     * Current store.
     */
    protected $_store;
    protected $_helper;    
    protected $_totalPackageWeight;
    protected $_totalPackageItemQty;

    protected $_currencyModel;
    /**
     * Module debug mode enabled
     *
     * @var bool
     */
    protected $_debugModeEnabled;

    /**
     * Tax sum for particular groups
     *
     * @var array
     */
    protected $_taxGroupsSummary = array();
    const TAX_GROUPS_SUM_AMOUNT = "tax_amount";
    const TAX_GROUPS_SUM_AMOUNT_BASE = "tax_amount_base";
    const TAX_PERCENT_DECIMAL_PLACES = 4;

    /**
     * Input data updated by information from database
     */
    protected $_dataUpdated;   

    protected function _getCurrencyModel()
    {
        if(!$this->_currencyModel)
            $this->_currencyModel = Mage::getModel('koongoorderapi/currency');
        return $this->_currencyModel;
    }

    /**
     * Get available attributes of order API resource
     *
     * @param string $userType
     * @param string $operation
     * @return array
     */
    public function getAvailableAttributes($userType, $operation)
    {
        $attributes = $this->getAvailableAttributesFromConfig();
        return $attributes;
    }

    /**
     * Check if order was already inserted
     *
     * @param Array $orderData
     * @return bool False if not exists or order entity is order exists.
     */
    protected function _getOrderByKoongoChannelId($orderData)
    {        
        $channelOrderId = null;
        if(isset($orderData['header']['channel_order_id']))
            $channelOrderId = $orderData['header']['channel_order_id'];     
           
        if(!$channelOrderId)
            return false;
            
        $order = Mage::getModel('sales/order')->load($channelOrderId,"koongo_channel_order_id");

        if(!$order->getId())
            return false;
        else
            return $order;
    }

    /**
     * Create or load customer account, if this feature is enabled
     */
    protected function _createCustomerAccount($customerData)
    {
        $canCreateCustomer = $this->_helper()->getCustomerConfig(Nostress_Koongoorderapi_Helper_Data::PARAM_CREATE_CUSTOMER_DURING_IMPORT,false, false, $this->_getStoreId());
        
        if(!$canCreateCustomer)
            return null;

        $websiteId = $this->_getWebsiteId();
        /** @var $customer Mage_Customer_Model_Customer */
        $customer = Mage::getModel("customer/customer")->setWebsiteId($websiteId);
        $customer->loadByEmail($customerData['email']);

        if($customer->getId())
            return $customer;
         
         try{
            $customerGroup = $this->_helper()->getCustomerConfig(Nostress_Koongoorderapi_Helper_Data::PARAM_CUSTOMER_GROUP_ID,false, false, $this->_getStoreId());

            $customer->setStore($this->_getStore())
             ->setFirstname($customerData['firstname'])
             ->setLastname($customerData['lastname'])
             ->setEmail($customerData['email'])
             ->setPhone($this->_getArrayField('phone', $customerData, "-"))
             ->setGroupId($customerGroup);             
        
            if(isset($customerData['sex']) && !empty($customerData['sex']))
                $customer->setGender($customerData['sex']== "M"?1:2); //set male or female

            $randomPass = $customer->generatePassword();
            $customer->setPassword($randomPass); 

            //Create comfirmed customer
            $customer->save();
            $customer->setConfirmation(null);
            $customer->save();
            return $customer;
        }
        catch(Exception $e)
        {
            $this->_critical($this->_helper()->__("Customer generation error: %s", $e->getMessage()),Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }
    }    

    /**
     * Assign customer to quote. If customer is not present, then create guest customer.
     *
     * @param Mage_Sales_Model_Quote $quote
     * @param Mage_Customer_Model_Customer $customer
     * @param Array $customerData
     * @return Mage_Sales_Model_Quote
     */
    protected function _assignCustomer($quote, $customer, $customerData)
    {
        if(isset($customer))
            $quote->assignCustomer($customer);
        else
        {   //Assign guest customer
            $quote->setCustomerIsGuest(true)
                ->setCustomerEmail($customerData['email'])
                ->setCustomerFirstname($customerData['firstname'])
                ->setCustomerLastname($customerData['lastname'])                
                ->setCustomerGroupId(Mage_Customer_Model_Group::NOT_LOGGED_IN_ID);                
        }
        return $quote->save();
    }

    protected function _initCurrency($quote, $orderCurrencyCode){
        $currencyModel = $this->_getCurrencyModel();
        $currencyModel->setStore($this->_getStore());

        if($currencyModel->currencyEqualsBase($orderCurrencyCode))
           $quote->getStore()->setCurrentCurrency($this->_getStore()->getBaseCurrency());
        else
        {            
            $rate = $currencyModel->getCurrencyConversionRateFromBase($orderCurrencyCode);             
            if($rate == 0)
                $this->_critical($this->_helper()->__("Currency %s is not defined or allowed in store.", $orderCurrencyCode),Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
            else
            {
                $currency = Mage::getModel('directory/currency')->load($orderCurrencyCode);
                $quote->getStore()->setCurrentCurrency($currency);                
                $currencyModel->setCurrencyForConversion($orderCurrencyCode);
            }
        }
        return $quote;
    }
    /**
     * Add address to quote and customer (if customer exists)
     *
     * @param Mage_Sales_Model_Quote $quote
     * @param Array $address Adress data.
     * @param Bool $isBilling Is billing ot not(= is shipping address).
     * @param string $customerPhone Customer telephone number.
     * @param Mage_Customer_Model_Customer $customer
     * @return Mage_Sales_Model_Quote Updated Quote.
     */
    protected function _addAddress($quote, $address, $isBilling, $customerPhone = "", $customer = null)
    {        
        $street = $this->_getArrayField('street_line1',$address,"");
        $streetLine2 = $this->_getArrayField('street_line2',$address,"");
        if(!empty($streetLine2))
            $street .= ", ".$streetLine2;

        //Modification for Tric GLS module - Pakkeshop string must have char \n as prefix, otherwise shipping labels can't be print
        if(strpos($street,"Pakkeshop") !== false)
            $street = preg_replace("/,\s*Pakkeshop/","\nPakkeshop",$street);

        $addressData = array(
            'prefix'    => "",
            'firstname' => $this->_getArrayField('firstname',$address,"-"),
            'middlename' => "",
            'lastname' => $this->_getArrayField('lastname',$address,""),
            'suffix' => '',
            'company' => $this->_getArrayField('company',$address,""),
            'street' => $street,
            'city' => $this->_getArrayField('city',$address,""),
            'country_id' => $this->_getArrayField('country_code',$address,""),
            'region' =>  $this->_getArrayField('state',$address,""),
            'postcode' => $this->_getArrayField('postcode',$address,""),
            'telephone' => $this->_getArrayField('phone',$address,""),
            'fax' => "",
            'vat_id' => "",            
        );

        //Customer phone can't be saved to customer entity
        if(empty($addressData['telephone']))
            $addressData['telephone'] = $customerPhone;        

        //Save address and check name if customer exists
        if(isset($customer))
        {
            $addressData['save_in_address_book'] = 1;
            if(empty($addressData['lastname']) && $addressData['firstname'] == "-")                
            {
                $addressData['lastname'] = $customer->getLastname();
                $addressData['firstname'] = $customer->getFirstname();
            }

            $addressModel = Mage::getModel("customer/address")->setData($addressData)
                            ->setCustomerId($customer->getId());
            if($isBilling)    
                $addressModel->setIsDefaultBilling('1');
            else
                $addressModel->setIsDefaultShipping('1');
            $addressModel->save();
        }
        
        if($isBilling)
        {    
            $billingAddress = $quote->getBillingAddress();
            $billingAddress->addData($addressData);
            $billingAddress->setShouldIgnoreValidation(true); //Ignore validation in order to insert address without state
            $billingAddress->save();
        }
        else
        {
            $shippingAddress = $quote->getShippingAddress();
            $shippingAddress->addData($addressData);
            $shippingAddress->setShouldIgnoreValidation(true);//Ignore validation in order to insert address without state

            $parcelShopId = $this->_getArrayField('parcel_shop_id',$address);                       
            if(isset($parcelShopId))
            {    
                $quote->setData("gls_pakkeshop",$parcelShopId); //Insertion of Pakkeshop for Tric GLS module
                $shippingAddress->setDroppoint($parcelShopId);
            }
            $shippingAddress->save();
        }

        return $quote;
    }

    protected function _addItems($quote, $items, $customer)
    {
        $this->_resetTotalPackageWeight();
        $this->_resetTotalPackageItemQty();

        $storeId = $this->_getStoreId();
        
        $catalogPricesIncludesTax = $this->_helper()->getConfig(Mage_Tax_Model_Config::CONFIG_XML_PATH_PRICE_INCLUDES_TAX,false, false, $this->_getStoreId());

        $subtotalExclTax = 0;
        $subtotalInclTax = 0;
        $taxAmount = 0;

        foreach ($items as $key => $item) 
        {
            $product = Mage::getModel('catalog/product')->setStoreId($storeId)->load($item['product_id']);            
            $this->_addToTotalPackageWeight($product->getWeight()*$item['qty']);
            $this->_addToTotalPackageItemQty($item['qty']);

            // Update tax_rate, tax and price excl tax if missing
            if(!$this->_getArrayField("base_excl_tax", $item['price']) || !$this->_getArrayField("tax_rate", $item['price']) || !$this->_getArrayField("tax", $item['price']))
            {
                $item = $this->_updateItemPriceAttribs($item, $key, $quote, $product, $customer);                
            }                                                                      
            
            $itemTotalInclTax = $item['price']['base_incl_tax']*$item['qty'];
            $itemTotalTax = $item['price']['tax'];

            $subtotalExclTax += $itemTotalInclTax - $itemTotalTax;
            $subtotalInclTax += $itemTotalInclTax;
            $taxAmount += $itemTotalTax;
            
            if($catalogPricesIncludesTax)
            {
                //System predpoklada ze vkladana cena obsahuje defaultni DPH pro store
                //Proto provere price/1.xx aby spocital cenu bez DPH
                //Predpocitame mu cenu tak aby po jeho vypoctu vznikla spravna cena bez DPH (podle tax rate dane adresou zakaznika)
                $storeTaxRate = $this->_getStoreTaxRate($product->getTaxClassId(), $quote->getShippingAddress(), $quote->getBillingAddress(), $customer);
                $price = (float) ($item['price']['base_incl_tax'] / (1+$item['price']['tax_rate']))*(1+$storeTaxRate);
            }
            else 
                $price = (float) $item['price']['base_excl_tax'];        
            
            //Insert product to quote
            try {
                $paramaters = new Varien_Object();
                $paramaters->setQty($item['qty']);

                $addProductResult = $quote->addProduct($product, $paramaters);    
                //If add product fails then string might be returned
                if(is_string ($addProductResult) == true)
                    throw new Exception($this->_helper()->__("Quote item creation failed. Reason").": ".$addProductResult);                
                
                $quoteItem = $quote->getItemByProduct($product);                 
                $quoteItem->setOriginalCustomPrice($price);                               
                
                //Quote created by admin
                $quoteItem->getProduct()->setIsSuperMode(true);                           
            }
            catch (Exception $e) {
                $message = $this->_helper()->__("Order creation failed due to following reason: %s", $e->getMessage());
                $this->_helper()->log($message);
                $this->_critical($message, Mage_Api2_Model_Server::HTTP_BAD_REQUEST);                
            }
        }

        $this->_updateTotals($subtotalExclTax, $subtotalInclTax, $taxAmount);           
        return $quote;
    }

    /**
     * Correct quote values according to imported order data
     *
     * @param Mage_Sales_Model_Order_Quote $quote
     * @param Array $orderData
     * @return void
     */
    protected function _correctQuote($quote, $orderData)
    {
        $currencyModel = $this->_getCurrencyModel(); 
        $storeId = $this->_getStoreId();    
        foreach ($orderData['items'] as $item) 
        {
            $product = Mage::getModel('catalog/product')->setStoreId($storeId)->load($item['product_id']);    
            $quoteItem = $quote->getItemByProduct($product);            
            
            $rowTotalInclTax = $item['price']['base_incl_tax']*$item['qty'];
            $baseRowTotalInclTax = $currencyModel->convertPriceToStoreCurrency($rowTotalInclTax);
            $quoteItem->setData('row_total_incl_tax',$rowTotalInclTax);
            $quoteItem->setData('base_row_total_incl_tax',$baseRowTotalInclTax);
            
            $priceInclTax = $item['price']['base_incl_tax'];
            $basePriceInclTax = $currencyModel->convertPriceToStoreCurrency($priceInclTax);
            $quoteItem->setData('price_incl_tax', $priceInclTax);
            $quoteItem->setData('base_price_incl_tax', $basePriceInclTax);
            
            $rowTotalTax = $item['price']['tax'];
            $baseRowTotaTax = $currencyModel->convertPriceToStoreCurrency($rowTotalTax);
            $quoteItem->setData('tax_amount',$rowTotalTax);
            $quoteItem->setData('base_tax_amount',$baseRowTotaTax);
            $quoteItem->setData('tax_percent',$item['price']['tax_rate']*100);
            $this->_updateTaxGroupsSummary($item['price']['tax_rate'], $rowTotalTax, $baseRowTotaTax);

            $rowTotalExclTax = $rowTotalInclTax - $rowTotalTax;
            $baseRowTotalExclTax = $currencyModel->convertPriceToStoreCurrency($rowTotalExclTax);
            $quoteItem->setData('base_row_total',$baseRowTotalExclTax);            
            $quoteItem->setData('row_total',$rowTotalExclTax);

            $price = $item['price']['base_excl_tax'];
            $basePrice = $currencyModel->convertPriceToStoreCurrency($price);            
            $quoteItem->setData('price', $price);            
            $quoteItem->setData('original_custom_price',$price);
            $quoteItem->setData('base_price',$basePrice);
        }

        $shippingData = $orderData['shipping']; 
        $shippingAddress = $quote->getShippingAddress();

        $shippingTax = $shippingData['tax'];
        $baseShippingTax = $currencyModel->convertPriceToStoreCurrency($shippingTax);
        $shippingAddress->setData('shipping_tax_amount',$shippingTax);
        $shippingAddress->setData('base_shipping_tax_amount',$baseShippingTax);
        $this->_updateTaxGroupsSummary($shippingData['tax_rate'], $shippingTax, $baseShippingTax);

        $shippingAmount = $shippingData['cost_excl_tax'];
        $baseShippingAmount = $currencyModel->convertPriceToStoreCurrency($shippingAmount);
        $shippingAddress->setData('shipping_amount',$shippingAmount);
        $shippingAddress->setData('base_shipping_amount',$baseShippingAmount);
        
        $shippingInclTax = $shippingData['cost_incl_tax'];
        $baseShippingInclTax = $currencyModel->convertPriceToStoreCurrency($shippingInclTax);
        $shippingAddress->setData('shipping_incl_tax',$shippingInclTax);
        $shippingAddress->setData('base_shipping_incl_tax',$baseShippingInclTax);        

        $totalTax = $shippingData['tax']+$orderData['order_total']['tax'];
        $baseTotalTax = $currencyModel->convertPriceToStoreCurrency($totalTax);
        $shippingAddress->setData('tax_amount', $totalTax);
        $shippingAddress->setData('base_tax_amount', $baseTotalTax);
    
        $subtotalExclTax = $orderData['order_total']['subtotal_excl_tax'];
        $baseSubtotalExclTax = $currencyModel->convertPriceToStoreCurrency($subtotalExclTax);
        $shippingAddress->setData('subtotal', $subtotalExclTax);
        $shippingAddress->setData('base_subtotal', $baseSubtotalExclTax);

        $subtotalInclTax = $orderData['order_total']['subtotal_incl_tax'];
        $baseSubtotalInclTax = $currencyModel->convertPriceToStoreCurrency($subtotalInclTax);
        $shippingAddress->setData('subtotal_incl_tax', $subtotalInclTax);
        $shippingAddress->setData('base_subtotal_incl_tax', $baseSubtotalInclTax);

        $grandTotal = $subtotalInclTax + $shippingInclTax;
        $baseGrandTotal = $baseSubtotalInclTax + $baseShippingInclTax;
        $shippingAddress->setData('grand_total', $grandTotal);
        $shippingAddress->setData('base_grand_total', $baseGrandTotal);

        $quote->save();
        return $quote;
    }
    
    /**
     * Correct invoice values according to imported order data
     *
     * @param Mage_Sales_Model_Order_Invoice $invoice
     * @param Mage_Sales_Model_Order $order
     * @return void
     */
    protected function _correctInvoice($invoice, $order)
    {
        $invoice->setGrandTotal($order->getGrandTotal());
        $invoice->setBaseGrandTotal($order->getBaseGrandTotal());
    }

    /**
     * Correct credit memo according to imported order data
     *
     * @param Mage_Sales_Model_Order_Creditmemo $creditMemo
     * @param Mage_Sales_Model_Order $order
     * @return void
     */
    protected function _correctCreditMemo($creditmemo, $order)
    {
        $creditmemo->setGrandTotal($order->getGrandTotal());
        $creditmemo->setBaseGrandTotal($order->getBaseGrandTotal());
    }

    /**
     * Get tax rate based on addresses, store and tax class id
     *
     * @param Int $taxClassId
     * @param Mage_Sales_Model_Address $billingAddress
     * @param Mage_Sales_Model_Address $shippingAddress
     * @return void
     */
    protected function _getTaxRate($taxClassId, $shippingAddress, $billingAddress, $customer)
    {
        $taxCalculation =  Mage::getModel('tax/calculation');
        if(isset($customer))
            $taxCalculation->setCustomer($customer);
        $store = $this->_getStore();        
        $rateRequest = $taxCalculation->getRateRequest($shippingAddress, $billingAddress, null, $store);  
        $percent = $taxCalculation->getRate($rateRequest->setProductClassId($taxClassId));
       //$this->_critical($percent, Mage_Api2_Model_Server::HTTP_BAD_REQUEST);  
        return $percent/100;
    }

     /**
     * Get store tax rate based on addresses, store and tax class id
     *
     * @param Int $taxClassId
     * @param Mage_Sales_Model_Address $billingAddress
     * @param Mage_Sales_Model_Address $shippingAddress
     * @return void
     */
    protected function _getStoreTaxRate($taxClassId, $shippingAddress, $billingAddress, $customer)
    {
        $taxCalculation =  Mage::getModel('tax/calculation');
        if(isset($customer))
            $taxCalculation->setCustomer($customer);
        $store = $this->_getStore();        
        $rateRequest = $taxCalculation->getRateRequest($shippingAddress, $billingAddress, null, $store);  
        $percent = $taxCalculation->getStoreRate($rateRequest->setProductClassId($taxClassId), $store);
        return $percent/100;
    }

    /**
     * Caculate tax_rate, tax and price excl tax
     *
     * @param Array $item
     * @param Array key $itemKey
     * @param Mage_Sales_Model_Quote $quote
     * @param Mage_Catalog_Model_Product $product
     * @return void
     */
    protected function _updateItemPriceAttribs($item, $itemKey, $quote, $product, $customer)
    {
        if(!$this->_getArrayField("tax_rate", $item['price']))
        {            
            $item['price']['tax_rate'] = $this->_getTaxRate($product->getTaxClassId(), $quote->getShippingAddress(), $quote->getBillingAddress(), $customer);
        }        

        if(!$this->_getArrayField("base_excl_tax", $item['price']))
        {            
            $item['price']['base_excl_tax'] = round($item['price']['base_incl_tax']/ (1+$item['price']['tax_rate']), 2);
        }

        if(!$this->_getArrayField("tax", $item['price']))
        {
            $item['price']['tax'] = round($item['price']['base_excl_tax']* $item['price']['tax_rate']*$item['qty'], 2);
        }

        $this->_updateData($item, "order", "items",$itemKey); 

        return $item;
    }

    /**
     * Update order totals
     *
     * @param Float $subtotalExclTax
     * @param Float $subtotalInclTax
     * @param Float $taxAmount
     * @return void
     */
    protected function _updateTotals($subtotalExclTax, $subtotalInclTax, $taxAmount)
    {
        $dataUpdated = $this->_getDataUpdated();
        $orderTotals = $dataUpdated['order']['order_total'];

        //Update total according to calculated amounts from items
        //if(!$this->_getArrayField("tax", $orderTotals))
        $orderTotals['tax']  = $taxAmount;

        //if(!$this->_getArrayField("subtotal_incl_tax", $orderTotals))
        $orderTotals['subtotal_incl_tax']  = $subtotalInclTax;
        
        //if(!$this->_getArrayField("subtotal_excl_tax", $orderTotals))
        $orderTotals['subtotal_excl_tax']  = $subtotalExclTax;

        $dataUpdated['order']['order_total'] = $orderTotals;
        $dataUpdated = $this->_setDataUpdated($dataUpdated);
    }

    /**
     * Set shipping method and assign price
     *
     * @param Mage_Sales_Model_Quote $quote
     * @param Array $orderData
     * @return Mage_Sales_Model_Quote
     */    
    protected function _addShippingMethod($quote, $orderData, $customer)
    {    
        $shippingData = $orderData['shipping'];
        $shippingTaxClass = $this->_helper()->getConfig(Mage_Tax_Model_Config::CONFIG_XML_PATH_SHIPPING_TAX_CLASS,false, false, $this->_getStoreId());    
        if(!$this->_getArrayField("tax_rate", $shippingData))
        {
            $shippingData['tax_rate'] = $this->_getTaxRate($shippingTaxClass, $quote->getShippingAddress(), $quote->getBillingAddress(), $customer);

            if(!is_numeric($shippingData['tax_rate']))
                $shippingData['tax_rate'] = 0;
        }
        
        if(!$this->_getArrayField("cost_excl_tax", $shippingData))
        {
            $shippingData['cost_excl_tax'] = round($shippingData['cost_incl_tax'] / (1+$shippingData['tax_rate']), 2);
        }

        if(!$this->_getArrayField("cost_incl_tax", $shippingData))        
        {
            $shippingData['cost_incl_tax'] = round($shippingData['cost_excl_tax'] * (1+$shippingData['tax_rate']), 2);
        }

        if(!$this->_getArrayField("tax", $shippingData))        
        {
            $shippingData['tax'] = round($shippingData['cost_excl_tax'] * $shippingData['tax_rate'], 2);
        }
             
        $this->_updateData($shippingData, "order", "shipping");        
        
        $shippingCostIncludesTax = $this->_helper()->getConfig(Mage_Tax_Model_Config::CONFIG_XML_PATH_SHIPPING_INCLUDES_TAX,false, false, $this->_getStoreId());
        
        if($shippingCostIncludesTax)
        {
            //System predpoklada ze vkladana cena obsahuje defaultni DPH pro store
            //Proto provere price/1.xx aby spocital cenu bez DPH
            //Predpocitame mu cenu tak aby po jeho vypoctu vznikla spravna cena bez DPH (podle tax rate dane adresou zakaznika)
            $storeTaxRate = $this->_getStoreTaxRate($shippingTaxClass, $quote->getShippingAddress(), $quote->getBillingAddress(), $customer);
            $shippingCost = (float) ($shippingData['cost_incl_tax']/(1+$shippingData['tax_rate']))*(1+$storeTaxRate);
        }
        else 
            $shippingCost = (float) $shippingData['cost_excl_tax'];

        //Prepare shipping cost for collect rates on Nostress_Koongoorderapi_Model_Carrier_Koongo       
        Mage::register("koongoorderapi_shipping_cost", $shippingCost);
        $shippingMethod = $this->_getAvailableShippingMethod($quote, $orderData);

        $shippingAddress = $quote->getShippingAddress();
		$shippingAddress->setShippingMethod($shippingMethod)
			->setCollectShippingRates(false) //Rates are collected during shipping methods check
			->collectTotals();
        
        return $quote;
    }

	    /**
     * Get available shipping method
     *
     * @param Mage_Sales_Model_Quote $quote
     * @param Array $orderData
     */ 
    protected function _getAvailableShippingMethod($quote, $orderData)
    {
        $currencyModel = $this->_getCurrencyModel(); 
        $totals = $orderData['order_total'];
		$shippingAddress = $quote->getShippingAddress();		
        $shippingAddress->setAllItems($quote->getAllItems());
        
        $subtotalInclTax = $totals['subtotal_incl_tax'];
        $baseSubtotalInclTax = $currencyModel->convertPriceToStoreCurrency($subtotalInclTax);
        $shippingAddress->setBaseSubtotal($baseSubtotalInclTax);
        $shippingAddress->setBaseSubtotalWithDiscount($baseSubtotalInclTax);
        $shippingAddress->setSubtotal($subtotalInclTax);
		$shippingAddress->setSubtotalWithDiscount($subtotalInclTax);
		//setBaseVirtualAmount = 0 ?
        $shippingAddress->setWeight($this->_getTotalPackageWeight());
		$shippingAddress->setFreeMethodWeight(0);
		$shippingAddress->setFreeShipping(0);
		$shippingAddress->setItemQty($this->_getTotalPackageItemQty());
		
		//Calculate shipping rates for address
		$shippingAddress->setCollectShippingRates(true);
		$shippingAddress->collectShippingRates();
		$rates = $shippingAddress->getShippingRatesCollection();        

        $parcelShopId = null;
        if(isset($orderData['shipping_address']['parcel_shop_id']))
            $parcelShopId = $orderData['shipping_address']['parcel_shop_id'];

        $carrierMethodList = array();
		$carrierMethodTitleList = array();

		foreach($rates as $rate)
		{			
            $carrierCode = $rate->getCarrier();
            $carrierTitle = $rate->getCarrierTitle();
            $method = $rate->getMethod();
			$methodTitle = $rate->getMethodTitle();			
            
            $methodTitleLower = strtolower($methodTitle);            
            if(!isset($parcelShopId) && strpos($methodTitleLower,"pakkeshop") !== false)
                continue;
            if(isset($parcelShopId) && strpos($methodTitleLower,"pakkeshop") === false)
                continue;

            $carrierMethodList[$carrierCode."_".$method] =  $methodTitle;
			$carrierMethodTitleList[$carrierTitle." - ".$methodTitle] =  $carrierCode."_".$method;			
        }

        //Check default shipping method
        $defaultShippingMethod =  $this->_helper()->getGeneralConfig(Nostress_Koongoorderapi_Helper_Data::PARAM_DEF_SHIPPING_METHOD, false, false, $this->_getStoreId());
        if(isset($carrierMethodList[$defaultShippingMethod]))
            return $defaultShippingMethod;
        
        $shippingMethodsModel = Mage::getModel("koongoorderapi/config_source_shippingmethods");
        $shippingMethodTitle = $shippingMethodsModel->getCarrierMethodTitle($defaultShippingMethod);
        if(isset($carrierMethodTitleList[$shippingMethodTitle]))
            return $carrierMethodTitleList[$shippingMethodTitle];
        
        //Check secondary shipping method
        $secondaryShippingMethod =  $this->_helper()->getGeneralConfig(Nostress_Koongoorderapi_Helper_Data::PARAM_SEC_SHIPPING_METHOD, false, false, $this->_getStoreId());
        if(isset($carrierMethodList[$secondaryShippingMethod]))
            return $secondaryShippingMethod;
        
        $shippingMethodTitle = $shippingMethodsModel->getCarrierMethodTitle($secondaryShippingMethod);
        // throw new Exception(json_encode($carrierMethodTitleList)."SSS2<br>".$shippingMethodTitle."XX".$carrierMethodTitleList[$shippingMethodTitle]);
        if(isset($carrierMethodTitleList[$shippingMethodTitle]))
            return $carrierMethodTitleList[$shippingMethodTitle];

        //Set Koongo as Shipping method
        return "koongo_koongo";
	}

    /**
     * Set payment method - paid on marketplace
     *
     * @param Mage_Sales_Model_Quote $quote
     * @return Mage_Sales_Model_Quote
     */
    protected function _addPaymentMethod($quote)
    {
        // Set custom payment method        
        $paymentData = array('method' => 'koongo');
        $quote->getPayment()->importData($paymentData);
        return $quote;
    }

    /**
     * Create and order from quote
     *
     * @param Mage_Sales_Model_Quote $quote
     * @return Mage_Sales_Model_Order
     */
    protected function _createOrderFromQuote($quote)
    {
        try {
            $service = Mage::getModel('sales/service_quote', $quote);
            $service->submitAll();
            $order = $service->getOrder();
        } catch (Exception $e) {
            $this->_critical($this->_helper()->__("Order creation failed. Reason:")." ".$e->getMessage(), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }

        unset($quote);
        unset($service);

        return $order;
    }

    /**
     * Update order indentification attributes 
     *
     * @param [type] $order
     * @param [type] $orderHeaderData
     * @return void
     */
    protected function _processOrderIds($order, $orderHeaderData)
    {        
        $marketplaceCode = $this-> _getArrayField("marketplace_code",$orderHeaderData); 

        $incrementIdFormat = $this->_helper()->getGeneralConfig(Nostress_Koongoorderapi_Helper_Data::PARAM_ORDER_INCREMENT_ID_FORMAT,false, false, $this->_getStoreId());          
        if(empty($incrementIdFormat) || $incrementIdFormat == Nostress_Koongoorderapi_Model_Config_Source_Incrementidformat::CHANNEL_MARKETPLACE_ORDER_ID)
        {
            $incrementId = $this->_helper()->generateUniqueOrderIncrementId($orderHeaderData['channel_code'], $orderHeaderData['channel_order_id'], $marketplaceCode);
            $order->setIncrementId($incrementId);
        }

        $order->setKoongoChannelOrderId($orderHeaderData['channel_order_id']);
        $order->setKoongoChannelCode($orderHeaderData['channel_code']);
        
        $channelLabel = ucfirst($orderHeaderData['channel_code']);
        if(isset($orderHeaderData['channel_label']))
            $channelLabel = $orderHeaderData['channel_label'];
        $order->setKoongoChannelLabel($channelLabel);
        
        if(isset($marketplaceCode))
            $order->setKoongoMarketplaceCode($marketplaceCode);        

        $order->save();        
    }

    /**
     * Update store order data
     *
     * @param [type] $order
     * @param [type] $orderHeaderData
     * @return void
     */
    protected function _updateStoreOrderData($order, $orderHeaderData)
    {
        $orderHeaderData["store_order_id"] = $order->getId();
        $orderHeaderData["store_order_name"] = $order->getIncrementId();
        $orderHeaderData["store_order_url"] = Mage::helper('adminhtml')->getUrl("adminhtml/sales_order/view", array('order_id'=> $order->getId()));
        
        $this->_updateData($orderHeaderData, "order", "header");        
    }

    /**
     * Process order items and save identificators in own order data
     *
     * @param Mage_Sales_Model_Order $order
     * @param Array $orderItemsData
     * @return void
     */
    public function _processOrderItemIds($order, $orderItemsData)
    {
         
        //Prepare mapping between product id and order item id
        $productIdOrderItemIdMap = array();
        $orderItems = $order->getAllItems();
        foreach ($orderItems as $orderItem) {
            $productIdOrderItemIdMap[$orderItem->getProductId()] = $orderItem->getItemId();
        }

        //Assign store order item ids to own order data
        foreach ($orderItemsData as $key => $item) 
        {
            if(isset($productIdOrderItemIdMap[$item['product_id']]))
                $orderItemsData[$key]['store_order_item_id'] = $productIdOrderItemIdMap[$item['product_id']];
        }
        $this->_updateData($orderItemsData, "order", "items");
    }

    /**
     * Create invoice for the order and set processing status
     *
     * @param Mage_Sales_Model_Order $order
     * @param Array $orderData
     * @return void
     */
    protected function _prepareInvoice($order, $orderData)
    {        
        try {
            if(!$order->canInvoice()) {
                throw new Exception($this->_helper()->__("Invoice cannot be created."));
            }             
            $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice();                        

            if (!$invoice->getTotalQty()) {
                throw new Exception($this->_helper()->__("Cannot create an invoice without order items."));
            }
             // Add comment to invoice
            $channelTitle = ucfirst($orderData['header']['channel_code']);
            if(isset($orderData['header']['channel_label']))
                $channelTitle = $orderData['header']['channel_label'];
            $invoice->addComment($this->_helper()->__("Payment done at %s", $channelTitle), false, true);            
            $this->_correctInvoice($invoice, $order);

            $invoice->register();            
            $transactionSave = Mage::getModel('core/resource_transaction')
                ->addObject($invoice)
                ->addObject($invoice->getOrder());             
            $transactionSave->save();            

            $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true, $this->_helper()->__("Invoice created automatically."));            
            $this->_addTransactionToPayment($order);            
            $order->save();

        } catch (Mage_Core_Exception $e) {
                throw new Exception($this->_helper()->__("Order created, but invoice creation failed. Reason:")." ".$e->getMessage());
        }
    }

    /**
     * Create payment transaction for the order
     *
     * @param Mage_Sales_Model_Order $order
     * @return void
     */
    protected function _addTransactionToPayment($order)
    {
        $payment = $order->getPayment();
        $message = Mage::helper('kaas')->__('Payment done at %s.',$order->getKoongoChannelLabel());
        $payment->setTransactionId($order->getId());
        $payment->addTransaction(Mage_Sales_Model_Order_Payment_Transaction::TYPE_CAPTURE, null, false, $message);
    }

    /**
     * Add comments to order
     *
     * @param Mage_Sales_Model_Order $order
     * @param Array $commentsData     
     * @return void
     */
    protected function _addComments($order, $commentsData)
    {        
        if(!isset($commentsData) || !is_array($commentsData))
            return;

        foreach ($commentsData as $key => $comment) {            
            $this->_addComment($order, $comment);
        }
                
        $order->save();
    }

    /**
     * Add comment to order
     *
     * @param Mage_Sales_Model_Order $order
     * @param Array $commentsData     
     * @return void
     * */
    protected function _addComment($order, $commentData)
    {
        if(!isset($commentData['message']))
            throw new Exception($this->_helper()->__("Comment message is missing."));

        $order->addStatusHistoryComment($commentData['message'],false);
    }

    /**
     * Add Shipments to order
     *
     * @param Mage_Sales_Model_Order $order
     * @param Array $shipmentsData     
     * @return void
     */
    protected function _addShipment($order, $shipmentData)
    {        
        if(!isset($shipmentData) || !is_array($shipmentData))
            $this->_critical($this->_helper()->__("Missing shippment data"), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);

        if(!isset($shipmentData["shipped_order_items"]) || empty($shipmentData["shipped_order_items"]))
            $this->_critical($this->_helper()->__("Missing shipped_order_items in shipment."), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        
        //Prepare item qty for each shipment item
        $productsQty=array();
        $orderId = $order->getId();
        foreach ($shipmentData["shipped_order_items"] as $key => $shipmentOrderItem) {
            $orderItemId = $this->_getArrayField("store_order_item_id",$shipmentOrderItem,"");
            if( empty( $orderItemId)) {
                $this->_critical($this->_helper()->__("store_order_item_id is missing for item"), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
            }

            $orderItem = Mage::getModel('sales/order_item')->load($orderItemId);
            
            if(!$orderItem->getId())
                $this->_critical($this->_helper()->__("Order item with id %s is missing.", $orderItemId), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);

            if($orderItem->getOrderId() != $orderId)
                $this->_critical($this->_helper()->__("Order id (%s) from order item differs from id of current order (%s) .", $orderItem->getOrderId(), $orderId), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);

            $itemQtyForShipping = $orderItem->getQtyOrdered() - $orderItem->getQtyShipped() - $orderItem->getQtyRefunded()
                - $orderItem->getQtyCanceled();

            $shipmentOrderItemQty = $this->_getArrayField('qty', $shipmentOrderItem);
            if(!isset($shipmentOrderItemQty))
                $this->_critical($this->_helper()->__("Order item #%s: Missing quantity in shipment item.",$orderItemId), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);                        

            if($shipmentOrderItemQty != $itemQtyForShipping)
                continue; //Item not available for shipment                
            
            $productsQty[$orderItemId] = $itemQtyForShipping;
        }

        if(empty($productsQty)) //Nothing remain to ship
        {
            $shipment = $this->_addTrackningNumberToExistingShipment($order, $shipmentData);             
            if($shipment)
                return $shipment->getId();
            
            $this->_critical($this->_helper()->__("Order items not available for shipping in requested quantity. Check order shipments in store."), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }

        if (!$order->canShip())
            $this->_critical($this->_helper()->__("Order cannot be shipped."), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);

        /* @var $shipment Mage_Sales_Model_Order_Shipment */
        $shipment = $order->prepareShipment($productsQty);
          
        if (!$shipment)
            $this->_critical($this->_helper()->__("Shipment creation failed."), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);

        $shipment->register();       
        $shipmentComment = $this->_composeComment($shipmentData);
        if(!empty($shipmentComment))
            $shipment->addComment($shipmentComment, false);

        $shipment->getOrder()->setIsInProcess(true);
        try {
            $transaction = Mage::getModel('core/resource_transaction')
                ->addObject($order)
                ->addObject($shipment)
                ->save();
            
            Mage::register("koongoorderapi_shipment_create_from_kaas", true);
            $this->_addTrackingNumber($shipment, $shipmentData);
        } catch (Mage_Core_Exception $e) {
            Mage::unregister("koongoorderapi_shipment_create_from_kaas");
            $this->_critical($this->_helper()->__("Shipment creation failed. Reason: %s", $e->getMessage()), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }
        
        Mage::unregister("koongoorderapi_shipment_create_from_kaas");

        return $shipment->getId();
    }

     /**
     * Check if tracking number is added and if not => add tracknig number to existing shipment
     *
     * @param Mage_Sales_Model_Order $order
     * @param Array $shipmentsData     
     * @return void
     */
    protected function _addTrackningNumberToExistingShipment($order, $shipmentData)
    {
        $shipmentCollection = Mage::getResourceModel('sales/order_shipment_collection')
                ->setOrderFilter($order)
                ->load();

        $trackingNumber = trim($this->_getArrayField('tracking_number', $shipmentData, ""));
        $lastShipment = null;
        
        //Check if tracknig number is already assigned
        foreach($shipmentCollection as $shipment)
        {
            $lastShipment = $shipment;            
            if(empty($trackingNumber)) //select current shipment
                return $lastShipment; //tracking number is not available, only return shipment

            foreach($shipment->getAllTracks() as $tracknum)
            {
                $currentTrackNum = trim($tracknum->getNumber());
                if(strcasecmp($trackingNumber, $currentTrackNum) == 0) //tracknig number already assigned
                    return $shipment;
            }               
        }

        if(!$lastShipment)
            return false;
            
        try {
            Mage::register("koongoorderapi_shipment_create_from_kaas", true);
            $this->_addTrackingNumber($lastShipment, $shipmentData);
        } catch (Mage_Core_Exception $e) {
            Mage::unregister("koongoorderapi_shipment_create_from_kaas");
            $this->_critical($this->_helper()->__("Shipment creation failed. Reason: %s", $e->getMessage()), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }        
        Mage::unregister("koongoorderapi_shipment_create_from_kaas");
        return $lastShipment;
    }

    /**
     * Refund all invoices from the order
     */
    protected function _prepareCreditMemo($order, $commentMessage, $creditmemoData = array())
    {        
        /** @var $service Mage_Sales_Model_Service_Order */
        $service = Mage::getModel('sales/service_order', $order);
        /** @var $creditmemo Mage_Sales_Model_Order_Creditmemo */
        $creditmemo = $service->prepareCreditmemo($creditmemoData);        
        $this->_correctCreditMemo($creditmemo, $order);                
        $creditmemo->setPaymentRefundDisallowed(true)->register();
        // add comment to creditmemo
        if (!empty($commentMessage)) {
            $creditmemo->addComment($commentMessage, false);
        }
        try {
            Mage::getModel('core/resource_transaction')
                ->addObject($creditmemo)
                ->addObject($order)
                ->save();
            
        } catch (Mage_Core_Exception $e) {
            $this->_critical($this->_helper()->__("Error during order cancelation. Reason: %s",$e->getMessage()), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }
        return $creditmemo->getIncrementId();
   }

    /**
     * Prepare comment for shipment
     *
     * @param Array $shipmentData
     * @return void
     */
    protected function _composeComment($shipmentData)
    {
        $shipmentComment = "";
        $value = $this->_getArrayField("carrier_name",$shipmentData,"");
        if(!empty($value))
            $shipmentComment = $this->_addShipmentCommentRow($shipmentComment, $shipmentData, "carrier_name", "Carrier name");
        else
        {
            $shipmentComment = $this->_addShipmentCommentRow($shipmentComment, $shipmentData, "carrier_code", "Carrier code");
        }

        $shipmentComment = $this->_addShipmentCommentRow($shipmentComment, $shipmentData, "tracking_number", "Tracking number");
        $shipmentComment = $this->_addShipmentCommentRow($shipmentComment, $shipmentData, "tracking_url", "Tracking url");
        $shipmentComment = $this->_addShipmentCommentRow($shipmentComment, $shipmentData, "delivery_method_name", "Delivery method name");
        $shipmentComment = $this->_addShipmentCommentRow($shipmentComment, $shipmentData, "delivery_method_code", "Delivery methodcode");
        $shipmentComment = $this->_addShipmentCommentRow($shipmentComment, $shipmentData, "delivery_method_description", "Delivery method description");

        return $shipmentComment;
    }

    /**
     * Add row for shipment comment
     *
     * @param String $shipmentComment
     * @param Array $shipmentData
     * @param String $index
     * @param String $label
     * @return String
     */
    protected function _addShipmentCommentRow($shipmentComment, $shipmentData, $index, $label)
    {
        $value = $this->_getArrayField($index,$shipmentData,"");
        if(!empty($value))
            $shipmentComment .= $this->_helper()->__($label).": ".$value."<br/>";
        return $shipmentComment;
    }
    /**
     * Add tracking number to shipment
     *
     * @param Mage_Sales_Model_Order_Shipment $shipment
     * @param Array $shipmentData
     * @return void
     */
    protected function _addTrackingNumber($shipment, $shipmentData)
    {
        $shipmentTrack = Mage::getModel('sales/order_shipment_track')
                 ->setShipment($shipment);
        $shippingMethodTitle = $this->_getArrayField('carrier_name', $shipmentData, "");
        if(empty($shippingMethodTitle))
            $shippingMethodTitle = $this->_getArrayField('carrier_code', $shipmentData, "");

        $deliveryMethoodName = $this->_getArrayField('delivery_method_name', $shipmentData, "");
        if(empty($deliveryMethoodName))
            $deliveryMethoodName = $this->_getArrayField('delivery_method_code', $shipmentData, "");

        if(!empty($shippingMethodTitle))
            $shippingMethodTitle .= " ";
        $shippingMethodTitle .=  $deliveryMethoodName;  

        $shipmentTrack->setTitle($shippingMethodTitle);
        $shipmentTrack->setNumber($this->_getArrayField('tracking_number', $shipmentData, ""));
        //tohle nefunguje 
        //$shipmentTrack->setUrl($this->_getArrayField('tracking_url', $shipmentData, ""));

        $carrierCode = $this->_getArrayField('carrier_code', $shipmentData, "");
        if(empty($carrierCode))
            $carrierCode = $this->_getArrayField('delivery_method_code', $shipmentData, "");
            
        $shipmentTrack->setCarrierCode($carrierCode);
        $shipmentTrack->setOrderId($shipment->getOrderId());
        $shipmentTrack->save();
    }

    protected function _helper()
    {
        if(!isset($this->_helper))
    	    $this->_helper = Mage::helper('koongoorderapi/order_data');
    	return $this->_helper;
    }
    
    protected function _log($message)
    {
        $this->_helper()->log($message);
    }

    /**
     * Debug message loging
     *
     * @param string $message
     * @return void
     */
    protected function _debug($message)
    {
        if($this->_isDebugModeEnabled())
            $this->_log($message);

    }

    protected function _isDebugModeEnabled()
    {
        if(!isset($this->_debugModeEnabled))
            $this->_debugModeEnabled = $this->_helper()->getGeneralConfig(Nostress_Koongoorderapi_Helper_Data::PARAM_DEBUG_MODE, false, false, Mage_Core_Model_App::ADMIN_STORE_ID);
        return $this->_debugModeEnabled;
    }

    protected function _setStore($store)
    {
        if(is_numeric($store))
            $this->_store = Mage::app()->getStore($store);
        else
            $this->_store = $store;
    }

    protected function _getStore()
    {
        return $this->_store;
    }

    protected function _getStoreId()
    {
        return $this->_getStore()->getStoreId();
    }

    protected function _getWebsiteId()
    {
        return $this->_getStore()->getWebsiteId();
    }

    protected function _getArrayField($index,$array, $default = null)
    {
        if(!is_array($array))
        {
            return $default;
        }
    	if(array_key_exists($index,$array))
    	{
    		return $array[$index];
    	}
    	else
    		return $default;
    }

    protected function _resetTotalPackageWeight()
    {
        $this->_totalPackageWeight = 0;
    }

    protected function _addToTotalPackageWeight($weight)
    {
        $this->_totalPackageWeight += $weight;
    }

    protected function _getTotalPackageWeight()
    {
        return $this->_totalPackageWeight;
    }

    protected function _resetTotalPackageItemQty()
    {
        $this->_totalPackageItemQty = 0;
    }

    protected function _addToTotalPackageItemQty($qty)
    {
        $this->_totalPackageItemQty += $qty;
    }

    protected function _getTotalPackageItemQty()
    {
        return $this->_totalPackageItemQty;
    }


     /**
     * Get updated order data.
     * E.g. product identificators might be added.
     */
    protected function _getDataUpdated()
    {
        return $this->_dataUpdated;
    }

    /**
     * Initiate updated order data.
     */
    protected function _setDataUpdated($data)
    {
        $this->_dataUpdated = $data;
    }
    
    /**
     * Update order data, which will be returned in response
     *
     * @param Array $data
     * @param String $index
     * @param String $subIndex
     * @param String $subSubIndex
     * @return void
     */
    protected function _updateData($data, $index, $subIndex = null, $subSubIndex = null)
    {
        if(!isset($subIndex))
            $this->_dataUpdated[$index] = $data;
        else if(!isset($subSubIndex))
            $this->_dataUpdated[$index][$subIndex] = $data;
        else
            $this->_dataUpdated[$index][$subIndex][$subSubIndex] = $data;
    }

    /**
     * Load order by real order id
     *
     * @param Int $realOrderId
     * @return Mage_Sales_Model_Order
     */
    protected function _loadOrder($realOrderId)
    {
        $order = Mage::getModel('sales/order')->load($realOrderId);        
        if($order->getId())
            return $order;

        $this->_critical($this->_helper()->__("Order with id %s doesn't exist.",$realOrderId), Mage_Api2_Model_Server::HTTP_INTERNAL_ERROR);
    
        return false;
    }

        /**
     * Add tax amount to given rate
     *
     * @param float $taxRate Tax rate
     * @param float $taxAmount Tax amount
     * @param float $taxAmountBase Tax amaount in base currency
     * @return void
     */
    protected function _updateTaxGroupsSummary($taxRate, $taxAmount, $taxAmountBase)
    {
        $taxPercent = round($taxRate*100, self::TAX_PERCENT_DECIMAL_PLACES);
        if(!isset($this->_taxGroupsSummary[$taxPercent]))
        {
            $this->_taxGroupsSummary[$taxPercent] = array(  self::TAX_GROUPS_SUM_AMOUNT => 0,
                                                        self::TAX_GROUPS_SUM_AMOUNT_BASE => 0);
        }
        
        $this->_taxGroupsSummary[$taxPercent][self::TAX_GROUPS_SUM_AMOUNT] += $taxAmount;
        $this->_taxGroupsSummary[$taxPercent][self::TAX_GROUPS_SUM_AMOUNT_BASE] += $taxAmountBase;
    }

    /**
     * Get tax group summary per tax percent
     *
     * @param float $taxPercent
     * @return void
     */
    protected function _getTaxGroupSummaryByPercent($taxPercent)
    {
        $taxPercent = round($taxPercent, self::TAX_PERCENT_DECIMAL_PLACES);                
        if(isset($this->_taxGroupsSummary[$taxPercent]))
            return $this->_taxGroupsSummary[$taxPercent];
        else
            return false;
    }

    /**
     * Correct sales order tax groups
     *
     * @param Mage_Sales_Model_Order $order
     * @return void
     */
    protected function _correctOrderTaxSummary($order)
    {
        $taxCollection = Mage::getModel('tax/sales_order_tax')->getCollection()->loadByOrder($order);
        foreach($taxCollection as $salesOrderTax)
        {
            $taxPercent = $salesOrderTax->getPercent();
            $taxGroup = $this->_getTaxGroupSummaryByPercent($taxPercent);            
            if($taxGroup)
            {
                if($taxGroup[self::TAX_GROUPS_SUM_AMOUNT])
                    $salesOrderTax->setAmount($taxGroup[self::TAX_GROUPS_SUM_AMOUNT]); 
                if($taxGroup[self::TAX_GROUPS_SUM_AMOUNT_BASE])
                {   
                    $salesOrderTax->setBaseAmount($taxGroup[self::TAX_GROUPS_SUM_AMOUNT_BASE]); 
                    $salesOrderTax->setBaseRealAmount($taxGroup[self::TAX_GROUPS_SUM_AMOUNT_BASE]); 
                }
                $salesOrderTax->save();
            }
        }
    }
}
