<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2018 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
 * API2 class for orders (Koongo)
 *
 * @category   Koongoorderapi
 * @package    Nostress_Koongoorderapi
 * @author     NoStress Commerce
 */
class Nostress_Koongoorderapi_Model_Api2_Order_Rest_Admin_V1 extends Nostress_Koongoorderapi_Model_Api2_Order
{
    /**
     * Create order
     *
     * @param array $data
     * @return string
     */
    protected function _create(array $data)
    {      
        $responseMessage = "";
        $responseStatus = Nostress_Koongoorderapi_Helper_Data::KOONGO_ORDER_COMMAND_STATUS_COMPLETE;

        $order = null;
        $skipProductAvailabilityCheck = false;
        if(isset($data['order']))
        {  
            $this->_debug("Check if order exists. InputData:".base64_encode(json_encode($data)));            
            $order = $this->_getOrderByKoongoChannelId($data['order']) ; 
            if($order)
                $skipProductAvailabilityCheck = true;           
        }

        $this->_validateData($data, $skipProductAvailabilityCheck);
        $data = $this->_getDataUpdated();
        $this->_setStore($data['store_id']); 

        if($order) //Order already exist
        {                 
            $this->_debug("Order already exists - store order id ".$order->getId());                        
            $responseMessage = $this->_helper()->__('Order already exists with store order id %s.',$order->getId());
            $responseStatus = Nostress_Koongoorderapi_Helper_Data::KOONGO_ORDER_COMMAND_STATUS_WARNING;            
        } 
        else
        {
            $this->_debug("Quote creation. Channel order id ".$data['order']['header']['channel_order_id']);
            /** @var Mage_Sales_Model_Quote $quote */
            $quote = Mage::getModel('sales/quote')->setStoreId($this->_getStoreId());
            //Quote super mode flag mean what we work with quote without restriction 
            //E.g. allow purchse of invisible or out of stock product (product availability is checked in validator)
            $quote->setIsSuperMode(true); 

            $quote = $this->_initCurrency($quote, $data['order']['order_total']['currency']);
            $customerData = $data['order']['customer'];
            $this->_debug("Create customer.");
            $customer = $this->_createCustomerAccount($customerData);
            $quote = $this->_assignCustomer($quote, $customer, $customerData);        
            $customerPhone = $this->_getArrayField('phone', $customerData, "-");
            $quote = $this->_addAddress($quote, $data['order']['billing_address'], true, $customerPhone, $customer);
            $quote = $this->_addAddress($quote, $data['order']['shipping_address'], false, $customerPhone, $customer);
            $quote = $this->_addItems($quote,$data['order']['items'], $customer);
            $data = $this->_getDataUpdated();

            $this->_debug("Quote add shipping and payment.");
            $quote = $this->_addShippingMethod($quote, $data['order'], $customer);
            $quote = $this->_addPaymentMethod($quote);
            $this->_debug("Quote collect totals.");
            $quote->collectTotals()->save();
            $data = $this->_getDataUpdated();     

            $quote = $this->_correctQuote($quote,$data['order']);

            $this->_debug("Create Order from Quote.");
            /** @var $order Mage_Sales_Model_Order */
            $order = $this->_createOrderFromQuote($quote);
            
            $this->_debug("Dispatch event: checkout_submit_all_after");            
            Mage::dispatchEvent('checkout_submit_all_after', array('order' => $order, 'quote' => $quote));
            
            $this->_debug("Update Order - store order id ".$order->getId());
            $this->_processOrderIds($order, $data['order']['header']);            
        }

        $this->_updateStoreOrderData($order, $data['order']['header']);
        $this->_processOrderItemIds($order, $data['order']['items']);
        $this->_correctOrderTaxSummary($order);
        
        //Order is ready, prepare invoice and return response        
        try {
            $this->_debug("Add comments");
            if(isset($data['order']['comments']))
                $this->_addComments($order, $data['order']['comments']);            
        }
        catch (Exception $e) 
        {
            if(!empty($responseMessage))
                $responseMessage .= " ";
            $responseMessage .= $e->getMessage();
            $responseStatus = Nostress_Koongoorderapi_Helper_Data::KOONGO_ORDER_COMMAND_STATUS_WARNING;
        } 

        try {
            $this->_debug("Prepare Invoice");
            $this->_prepareInvoice($order, $data['order']);
        }
        catch (Exception $e) 
        {
            if(!empty($responseMessage))
                $responseMessage .= " ";
            $responseMessage .= $e->getMessage();
            $responseStatus = Nostress_Koongoorderapi_Helper_Data::KOONGO_ORDER_COMMAND_STATUS_WARNING;
        }       
        
        unset($customer);

        $this->_returnResponse($responseStatus, $responseMessage);
    }

    /**
     * Validate incomming data for order insertion
     *
     * @param Array $data
     * @return void
     */
    protected function _validateData($data, $skipProductAvailabilityCheck = false)
    {
        /* @var $validator Nostress_Koongoorderapi_Model_Api2_Order_Validator_Order */
        $validator = Mage::getModel('koongoorderapi/api2_order_validator_order', array(
            'operation' => self::OPERATION_CREATE,
            'skipProductAvailabilityCheck' => $skipProductAvailabilityCheck
        ));

        $this->_debug("Validation start.");
        if (!$validator->isValidData($data)) {
            foreach ($validator->getErrors() as $error) {
                $this->_error($error, Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
            }
            $this->_critical(self::RESOURCE_DATA_PRE_VALIDATION_ERROR);
        }        
        //Order items ids and sku might be added
        $data = $validator->getDataUpdated();
        $this->_setDataUpdated($data);
    }

    protected function _returnResponse($responseStatus, $responseMessage)
    {
        $data = $this->_getDataUpdated();              
        $responseBody = $this->_helper()->prepareResponseBody($responseStatus, $responseMessage, $data);
        $this->_debug("Return response:".base64_encode($responseBody));  
        $this->getResponse()
            ->setBody($responseBody)
            ->sendResponse();
        exit;
    }
}
