<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2018 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
 * API2 order data validator (Koongo)
 *
 * @category   Koongoorderapi
 * @package    Nostress_Koongoorderapi
 * @author     NoStress Commerce
 */

class Nostress_Koongoorderapi_Model_Api2_Order_Validator_Order extends Mage_Api2_Model_Resource_Validator
{
    /**
     * The greatest decimal value which could be stored. Corresponds to DECIMAL (12,4) SQL type
     */
    const MAX_DECIMAL_VALUE = 99999999.9999;

    /**
     * Validation operation
     *
     * @var string
     */
    protected $_operation = null;

    /**
     * Skip product availability control
     *
     * @var boolean
     */
    protected $_skipProductAvailabilityCheck = false;

    /**
     * GTIN attribute code.          
     */
    protected $_gtinAttribute = null;

    /**
     * 
     * @var Nostress_Koongoorderapi_Helper_Data
     */
    protected $_helper;

    /**
     * Input data updated by information from database
     */
    protected $_dataUpdated;

    public function __construct($options)
    {
        if (!isset($options['operation']) || empty($options['operation'])) {
            throw new Exception($this->_helper()->__("Passed parameter 'operation' is empty."));
        }
        $this->_operation = $options['operation'];
        $this->_skipProductAvailabilityCheck = $options['skipProductAvailabilityCheck'];
    }

    /**
     * Get updated order data.
     * E.g. product identificators might be added.
     */
    public function getDataUpdated()
    {
        return $this->_dataUpdated;
    }

    /**
     * Initiate updated order data.
     */
    protected function _setDataUpdated($data)
    {
        $this->_dataUpdated = $data;
    }

    /**
     * Validate order data
     *
     * @param array $data
     * @return bool
     */
    public function isValidData(array $data)
    {           
        $this->_setDataUpdated($data);   

        try {
            $this->_validateStoreId($data);
            if(!$this->_isUpdate())
            {                
                if(!isset($data['order']))
                    $this->_critical($this->_helper()->__('Missing order data in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
                
                $this->_validateOrderHeader($data['order']);
                $this->_validateOrderTotal($data['order']);                
                $this->_validateUpdateProducts($data['order'], $data['store_id']);
                $this->_validateUpdateCustomer($data['order']);
                $this->_validateUpdateAddresses($data['order']);
                $this->_validateUpdateShipping($data['order']);
            }

            $isSatisfied = count($this->getErrors()) == 0;
        } catch (Mage_Api2_Exception $e) {
            $this->_addError($e->getMessage());
            $isSatisfied = false;
        }

        return $isSatisfied;
    }

    /**
     * Validate provided store id
     *
     * @param array $data
     * @return bool
     */
    protected function _validateStoreId($data)
    {
        if (!isset($data['store_id']) || empty($data['store_id'])) {
            $this->_critical($this->_helper()->__('Missing "store_id" in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }
        else
        {
            /** @var $store Mage_Core_Model_Store */            
            $store = Mage::getModel('core/store')->load($data['store_id']);
            if(!$store->getId()) //Can insert if store is not active //|| !$store->getIsActive()  
                $this->_critical($this->_helper()->__('Store with id %s is not available.',$data['store_id']), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }
    }

    /**
     * Validate order header
     *
     * @param Array $orderData
     * @return void
     */
    protected function _validateOrderHeader($orderData)
    {
        if (!isset($orderData['header']) || empty($orderData['header'])) {
            $this->_critical($this->_helper()->__('Missing order "header" in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }
        $headerData = $orderData['header'];

        if (!isset($headerData['channel_code']) || empty($headerData['channel_code'])) {
            $this->_critical($this->_helper()->__('Missing order "channel_code" in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }

        if (!isset($headerData['channel_order_id']) || empty($headerData['channel_order_id'])) {
            $this->_critical($this->_helper()->__('Missing order "channel_order_id" in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }
    }

    /**
     * Validate order total section
     *
     * @param Array $orderData
     * @return void
     */
    protected function _validateOrderTotal($orderData)
    {
        if (!isset($orderData['order_total']) || empty($orderData['order_total'])) {
            $this->_critical($this->_helper()->__('Missing order "order_total" in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }
        $totalData = $orderData['order_total'];

        if (!isset($totalData['currency']) || empty($totalData['currency'])) {
            $this->_critical($this->_helper()->__('Missing order "currency" in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }
    }

    /**
     * Validate and update provided order products
     *
     * @param array $data
     * @return bool
     */
    protected function _validateUpdateProducts($data, $storeId)
    {
        if (!isset($data['items']) || empty($data['items'])) {
            $this->_critical($this->_helper()->__('Missing order "items" in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }

        /** @var $stockItemModel Mage_Cataloginventory_Model_Stock_Item */
        $stockItemModel = Mage::getModel('cataloginventory/stock_item');
        $gtinAttribtue = $this->_getGtinAttribute($storeId); 

        foreach ($data['items'] as $itemKey => $item) {  
            $productId = isset($item['product_id'])?$item['product_id']:null;
            $productIdentificationAttributeCode = isset($item['product_identification_attribute_code'])?$item['product_identification_attribute_code']:null;
            $productIdentificationAttributeValue = isset($item['product_identification_attribute_value'])?$item['product_identification_attribute_value']:null;
            
            if($productIdentificationAttributeCode == "id")
                $productIdentificationAttributeCode = "entity_id";

            /** @var Mage_Catalog_Model_Product $_product */
            $product = $this->_helper()->loadCatalogProduct($storeId, $productId, $productIdentificationAttributeCode, $productIdentificationAttributeValue, $gtinAttribtue);            
            
            if(!$product->getId())
            {
                $idAttribute = null;
                $value = null;
                if(isset($productId))
                {
                    $idAttribute = "Product Id";
                    $value = $productId;
                }
                else if(isset($productIdentificationAttributeCode))
                {
                    $idAttribute = $productIdentificationAttributeCode;
                    $value = $productIdentificationAttributeValue;
                }                

                if(isset($idAttribute))
                    $this->_addError($this->_helper()->__('Product with %s = %s not found.',$idAttribute, $value), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);                                    
                else
                    $this->_addError($this->_helper()->__('Product not found.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);                                    
                continue;
            }            

            if (!isset($item['qty']) || empty($item['qty'])) {
                $this->_critical($this->_helper()->__('Missing order item "qty" for product #%s in request.', $product->getId()), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
            }
            
            if (!isset($item['price']['base_incl_tax']) || empty($item['price']['base_incl_tax'])) {
                if (!isset($item['price']['total']) || empty($item['price']['total']))                  
                    $this->_critical($this->_helper()->__('Missing order item "price - base_incl_tax and total" for product #%s in request.', $product->getId()), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);

                $item['price']['base_incl_tax'] = round($item['price']['total']/$item['qty'],2);
            }

            $item = $this->_updateProductData($product, $item, $itemKey);

            $stockItemModel->loadByProduct($product);
            $availableQty = $stockItemModel->getQty();        
            $minQty = $stockItemModel->getMinQty();

            $importBackorders = $this->_helper()->getGeneralConfig(Nostress_Koongoorderapi_Helper_Data::PARAM_IMPORT_BACKORDERS, false, false, $storeId);
            if(!$importBackorders && !$this->_skipProductAvailabilityCheck)
            {
                if (($availableQty - $item['qty']) < $minQty) 
                {
                    $this->_addError($this->_helper()->__('Requested quantity not available for Product #%s.',$product->getId()), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);                                    
                }            

                //Ignore is available for purchase condition
                // if (empty($item['parent_product_id']) && !$product->isSalable())
                // {                
                //     $this->_addError($this->_helper()->__('Product #%s is not available for purchase.',$product->getId()), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
                // }
            }

            $requiredOptions = $product->getRequiredOptions();
            if (!empty($requiredOptions))
                $this->_addError($this->_helper()->__('Product #%s contains required options. This feature is not supported.',$product->getId()), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);                                                                                              
        }

    }   
    
    /**
     * Update product identificators at _dataUpdated
     */
    protected function _updateProductData($product, $orderItem, $orderItemKey)
    {
        if(!isset($orderItem['product_id']) || empty($orderItem['product_id']))
            $orderItem['product_id'] = $product->getId();
        if(!isset($orderItem['sku']) || empty($orderItem['sku']))
            $orderItem['sku'] = $product->getSku();
        if(!isset($orderItem['ean']) || empty($orderItem['ean']))
        {
            $gtinAttribtue = $this->_getGtinAttribute();
            if(isset($gtinAttribtue))
            {    
                $ean = $product->getData($gtinAttribtue);            
                if(!empty($ean))
                    $orderItem['ean'] = $ean;
            }
        }
        
        if (!isset($orderItem['price']['total']) || empty($orderItem['price']['total']))
            $orderItem['price']['total'] = round($orderItem['price']['base_incl_tax'] * $orderItem['qty'],2);             

        //calcaulate parent product id and adjust variant id
        if($product->getTypeId() == "simple"){
            $orderItem['variant_id'] = $orderItem['product_id'];
            $parentIds = Mage::getModel('catalog/product_type_grouped')->getParentIdsByChild($product->getId());
            if(!$parentIds)
                $parentIds = Mage::getModel('catalog/product_type_configurable')->getParentIdsByChild($product->getId());
            if(isset($parentIds[0]))                   
                $orderItem['parent_product_id'] = $parentIds[0];                           
        }

        $this->_dataUpdated['order']['items'][$orderItemKey] = $orderItem; 
        return $orderItem;       
    }

    /**
     * Validate customer data. If firstname is missing, then add "-".
     *
     * @param [type] $data
     * @return void
     */
    protected function _validateUpdateCustomer($data)
    {
        if (!isset($data['customer']) || empty($data['customer'])) {
            $this->_critical($this->_helper()->__('Missing order "customer" in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }

        $customerData = $data['customer'];

        if (!isset($customerData['email']) || empty($customerData['email'])) {
            $this->_critical($this->_helper()->__('Missing order "customer email" in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }
        if (!isset($customerData['lastname']) || empty($customerData['lastname'])) {
            $this->_critical($this->_helper()->__('Missing order "customer lastname" in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }

        if (!isset($customerData['firstname']) || empty($customerData['firstname'])) {
            $customerData['firstname'] = "-";
        }

        if (!isset($customerData['phone']) || empty($customerData['phone'])) {
            $customerData['phone'] = "-";
        }

        $this->_dataUpdated['order']['customer'] = $customerData;
    }

    /**
     * Validate addresses. If billing address is missing use shipping address.
     *
     * @param Array $data
     * @return void
     */
    protected function _validateUpdateAddresses($data)
    {
        if (!isset($data['shipping_address']) || empty($data['shipping_address'])) {
            $this->_critical($this->_helper()->__('Missing order "shipping_address" in request.'), Mage_Api2_Model_Server::HTTP_BAD_REQUEST);
        }

        $this->_validateAddresses($data['shipping_address'], 'shipping address');

        if (!isset($data['billing_address']) || empty($data['billing_address'])) {
            $this->_dataUpdated['order']['billing_address'] = $data['shipping_address'];
        }  
        else
        {
            $this->_validateAddresses($data['billing_address'], 'billing address');
        }  
    }

    /**
     * Validate address for required fields
     *
     * @param [type] $data
     * @param [type] $addressType
     * @return void
     */
    protected function _validateAddresses($data, $addressType)
    {
        $missingFields = array();
        if(!isset($data['lastname']) || empty($data['lastname']))
        {
            $missingFields[] = 'last name';
        }

        if((!isset($data['street_line1']) && !isset($data['street_line2'])) || (empty($data['street_line1']) && empty($data['street_line2'])))
        {
            $missingFields[] = 'street';
        }

        if(!isset($data['city']) || empty($data['city']))
        {
            $missingFields[] = 'city';
        }

        if(!isset($data['postcode']) || empty($data['postcode']))
        {
            $missingFields[] = ' zip/postal code';
        }

        if(!isset($data['country_code']) || empty($data['country_code']))
        {
            $missingFields[] = 'country';
        }

        if(!empty($missingFields))
        {
            $this->_critical($this->_helper()->__('Please check %s information. Following fields are missing: %s',$addressType, implode(", ",$missingFields) ), Mage_Api2_Model_Server::HTTP_BAD_REQUEST); 
        }
    }

    /**
     * Add shipping for free if it is not defined.
     *
     * @param Array $data
     * @return void
     */
    protected function _validateUpdateShipping($data)    
    {
        if (!isset($data['shipping']) || empty($data['shipping'])) {
            $shippingData = array();
        }
        else {
            $shippingData = $data['shipping'];
        }

        if((!isset($shippingData['cost_incl_tax']) || empty($shippingData['cost_incl_tax'])) && (!isset($shippingData['cost_excl_tax']) || empty($shippingData['cost_excl_tax'])))
        {
            $shippingData['cost_excl_tax'] = 0;
            $shippingData['cost_incl_tax'] = 0;
        }

        $this->_dataUpdated['order']['shipping'] = $shippingData;
    }

    /**
     * Is update mode
     *
     * @return bool
     */
    protected function _isUpdate()
    {
        return $this->_operation == Mage_Api2_Model_Resource::OPERATION_UPDATE;
    }

    protected function _getGtinAttribute($storeId = null)
    {
        if(!isset($this->_gtinAttribute))
            $this->_gtinAttribute = $this->_helper()->getGeneralConfig(Nostress_Koongoorderapi_Helper_Data::PARAM_GTIN_ATTRIBUTE, false, false, $storeId);
        return $this->_gtinAttribute;
    }

    /**
     * Throw API2 exception
     *
     * @param string $message
     * @param int $code
     * @throws Mage_Api2_Exception
     */
    protected function _critical($message, $code)
    {
        throw new Mage_Api2_Exception($message, $code);
    }        
    
    protected function _helper()
    {
        if(!isset($this->_helper))
    	    $this->_helper = Mage::helper('koongoorderapi/order_data');
    	return $this->_helper;
    }
}
