<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2018 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
 * Webhook model for Koongo order api module
 *
 * @category   Koongoorderapi
 * @package    Nostress_Koongoorderapi
 * @author     NoStress Commerce
 */
class Nostress_Koongoorderapi_Model_Webhook extends Nostress_Koongoorderapi_Model_Abstract
{
	const WEBHOOK_TOPIC_CANCEL_ORDER = "orders.canceled";
	const WEBHOOK_TOPIC_ADD_SHIPMENT = "shipments.create";

    public function _construct() {
		parent::_construct ();
		$this->_init ('koongoorderapi/webhook');
	}

	/**
	 * Create payload for webhook request
	 *
	 * @param Nostress_Koongoorderapi_Model_Webhook_Event $event
	 * @return void
	 */
	public function preparePayloadForEvent($event)
	{
		$topic = $event->getTopic();
		$payload = array();
		switch($topic)
		{
			case self::WEBHOOK_TOPIC_CANCEL_ORDER:
				$payload = $this->_preparePayloadForOrderCancel($event);
				break;
			case self::WEBHOOK_TOPIC_ADD_SHIPMENT:
				$payload = $this->_preparePayloadForAddShipment($event);
				break;
		}
		return $payload;
	}

	/**
	 * Prepare payload for cancel order event
	 *
	 * @param Nostress_Koongoorderapi_Model_Webhook_Event $event
	 * @return void
	 */
	protected function _preparePayloadForOrderCancel($event)
	{
		$payload = json_decode($event->getParams(), true);
		$payload['store_order_id'] =  $event->getOrderId();
		return $payload;
	}

	/**
	 * Prepare payload for add shipment event
	 *
	 * @param Nostress_Koongoorderapi_Model_Webhook_Event $event
	 * @return void
	 */
	protected function _preparePayloadForAddShipment($event)
	{
		$params = json_decode($event->getParams(), true);
		if(!isset($params["shipment_id"]))
			return array();

		$shipment = Mage::getModel('sales/order_shipment')->load($params["shipment_id"]);
		
		if(!$shipment->getId())
		return array();

		$payload = array();		
		$payload['index'] =  $shipment->getId();
		$payload['store_order_id'] =  $event->getOrderId();
		$payload['store_shipment_id'] =  $shipment->getId();
		$payload['status'] =  "shipped";
		$payload["ship_date"] = $this->_helper()->formatDatetime($shipment->getCreatedAt());
		
		$track = false;
		if(isset($params["track_id"]))
		{
			$track = Mage::getModel("sales/order_shipment_track")->load($params["track_id"]);
			if(!$track->getId())
				$track = false;
		}		
		
		if($track) //Track from params is available
		{
			$payload = $this->_addTrackInfoToShippmentPayload($payload, $track);		
		}
		else //Track in params not defined or available
		{
			//Save just first tracking number
			foreach($shipment->getAllTracks() as $track)
			{			
				$payload = $this->_addTrackInfoToShippmentPayload($payload, $track);						
				break;
			}
		}

		$payloadItems = array();
		$numberOfItemsShipped = 0;
		$items = $shipment->getItemsCollection(); 
		foreach($items as $item)
		{
			$payloadItem = array();
			$payloadItem["index"] = $item->getId();
			$payloadItem["store_order_item_id"] = $item->getOrderItemId();			
			$qty = (int)$item->getQty();			
			$payloadItem["qty"] = $qty;
			$numberOfItemsShipped += $qty;	
			
			$payloadItem["product_id"] = $item->getProductId();
			$payloadItem["sku"] = $item->getSku();
			$payloadItem["name"] = $item->getName();
			$payloadItems[] = $payloadItem;
		}

		$payload['shipped_order_items'] = $payloadItems;	
		$payload["number_of_items_shipped"] = $numberOfItemsShipped;

		// echo json_encode($payload); exit();
		return $payload;
	}

	/**
	 * Add track data to payload
	 *
	 * @param array $payload
	 * @param Mage_Sales_Model_Order_Shipment_Track $track
	 * @return void
	 */ 
	protected function _addTrackInfoToShippmentPayload($payload, $track)
	{		
		$carrierCode = $track->getCarrierCode();
		$trackTitle = $track->getTitle();		
		
		if($track->isCustom() && !empty($trackTitle))
		{			
			$carrierName = $trackTitle;
		}
		else
		{
			$carrier = Mage::getSingleton('shipping/config')->getCarrierInstance($carrierCode);
			$carrierName = $carrier->getConfigData('title');		
		}

		$payload['carrier_name'] = $carrierName;
		$payload['carrier_code'] = $carrierCode;	

		$payload['delivery_method_name'] = $trackTitle;
		$payload['tracking_number'] = $track->getNumber();	

		return $payload;
	}
}
