<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2018 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
 * Webhook event manager model for Koongo order api module
 *
 * @category   Koongoorderapi
 * @package    Nostress_Koongoorderapi
 * @author     NoStress Commerce
 */
class Nostress_Koongoorderapi_Model_Webhook_Event_Manager extends Nostress_Koongoorderapi_Model_Abstract
{
    public function _construct() {
		parent::_construct ();
		$this->_init ('koongoorderapi/webhook_event_manager');
	}

	/**
	 * Add cancel order webhook events
	 *
	 * @param Mage_Sales_Model_Order $order
	 * @param String $cancelReason
	 * @param boolean $invoiceExists
	 * @return Array $eventIds
	 */
	public function addCancelOrderEvents($order, $cancelReason = "")
	{
		$topic = Nostress_Koongoorderapi_Model_Webhook::WEBHOOK_TOPIC_CANCEL_ORDER;
		return $this->addWebhookEvents($topic, $order->getId(), $order->getStoreId(), array("note" => $cancelReason));				
	}

	
	/**
	 * Add shipment webhook events
	 *
	 * @param Mage_Sales_Model_Shipment $shipment
	 * @param Mage_Sales_Model_Order_Shipment_Track $track
	 * @return Array $eventIds
	 */
	public function addShipmentEvents($shipment, $track)
	{
		$topic = Nostress_Koongoorderapi_Model_Webhook::WEBHOOK_TOPIC_ADD_SHIPMENT;
		$params = array("shipment_id" => $shipment->getId(),
						"track_id" => $track->getId());
		return $this->addWebhookEvents($topic, $shipment->getOrderId(), $shipment->getStoreId(), $params);				
	}

	/**
	 * Add webhook events to table
	 *
	 * @param String $topic Webhook topic
	 * @param Int $orderId
	 * @param Int $storeId
	 * @param Array $params
	 * @return Array $eventIds
	 */
	public function addWebhookEvents($topic, $orderId, $storeId, $params)
	{		
		$webhooks = $this->_getWebhooks($storeId, $topic);
		
		$eventIds = array();
		foreach ($webhooks as $webhook) {
			$event = $this->_createWebhookEvent($webhook, $orderId, $params);			
			$event->save();
			$eventIds[] = $event->getId();
		}		
		return $eventIds;
	}

	/**
	 * Duplicate event and adjsut parent event id and duplicity counter
	 * Returns new event
	 * 
	 * @param Nostress_Koongoorderapi_Model_Webhook_Event $event
	 * @return Nostress_Koongoorderapi_Model_Webhook_Event
	 */
	public function duplicateWebhookEvent($event)
	{
		$eventData = $event->getData();

		$newEvent = Mage::getModel('koongoorderapi/webhook_event');
		$newEvent->setData($eventData);
		$newEvent->setId(null);

		$parentEventId = $event->getParentEventId();
		if(empty($parentEventId))
			$parentEventId = $event->getId();

		$newEvent->setParentEventId($parentEventId);
		$counter = $event->getDuplicityCounter();
		$counter++;
		$newEvent->setDuplicityCounter($counter);
		$newEvent->setStatus(Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_PENDING);
		$newEvent->setMessage("");
		$now = $this->_helper()->getNow();
		$newEvent->setCreatedAt($now);
		$newEvent->setupdatedAt($now);
		return $newEvent;
	}

	/**
	 * Get webhooks by storeId and topic
	 *
	 * @param Int $storeId
	 * @param String $topic
	 * @return void
	 */
	protected function _getWebhooks($storeId, $topic)
	{
		$webhooksCollection = Mage::getModel('koongoorderapi/webhook')->getCollection();
		$select = $webhooksCollection->getSelect();
		$select->where("store_id = ?", $storeId);
		$select->where("topic = ?", $topic);

		return $webhooksCollection->load();
	}

	/**
	 * Create event for webhook
	 *
	 * @param Nostress_Koongoorderapi_Model_Webhook $webhook
	 * @param Int $orderId
	 * @param Array $params
	 * @return void
	 */
	protected function _createWebhookEvent($webhook, $orderId, $params)
	{
		$event = Mage::getModel('koongoorderapi/webhook_event');
		$event->setTopic($webhook->getTopic());
		$event->setWebhookId($webhook->getId());
		$event->setOrderId($orderId);
		$event->setStatus(Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_PENDING);
		$event->setParams(json_encode($params));
		$now = $this->_helper()->getNow();
		$event->setCreatedAt($now);
		$event->setupdatedAt($now);
		return $event;
	}

}
