<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2018 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
 * Webhook event Processor model for Koongo order api module
 *
 * @category   Koongoorderapi
 * @package    Nostress_Koongoorderapi
 * @author     NoStress Commerce
 */
class Nostress_Koongoorderapi_Model_Webhook_Event_Processor extends Nostress_Koongoorderapi_Model_Abstract
{
	/** Check events in processing state in given period */
	const EVENT_PROCESSING_CHECK_PERIOD = "-1 hours";
	/** Check and remove events older than diven number of days */ 	 
	const EVENT_REMOVAL_CHECK_PERIOD = "-14 days";
	/** Maximum number of event renews */
	const EVENT_RENEWAL_COUNTER_LIMIT = 5; 

	/** @var $_apiClient Nostress_Koongoorderapi_Model_Api_Client_Kaas */
	protected $_apiClient;

	/** @var Nostress_Koongoorderapi_Model_Webhook_Event_Manager */
	protected $_eventManager;

    public function _construct() {
		parent::_construct ();
		$this->_init ('koongoorderapi/webhook_event_processor');
	}

	/**
	 * Proces events
	 *
	 * @param Array $eventIds Event ids that should be processed (process all events if null)
	 * @return void
	 */
	public function processWebhookEvents($eventIds = null)
	{
		$this->_processPendingEvents($eventIds);
		$this->_renewProcessingEvents();
		$this->_removeOldEvents();
		$this->_renewErrorEvents();
	}

	/**
	 * Renew error events that has not reached maximum duplicity level
	 *
	 * @return void
	 */
	protected function _renewErrorEvents()
	{		
		$eventCollection = Mage::getModel('koongoorderapi/webhook_event')->getCollection();
		$select = $eventCollection->getSelect();		
		$select->where('status = ?',Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_ERROR);
		$select->where('duplicity_counter < ?',self::EVENT_RENEWAL_COUNTER_LIMIT);
		$eventCollection->load();	
		
		$eventManager = $this->_getManager();
		foreach ($eventCollection as $event) {
			$duplicateEvent = $eventManager->duplicateWebhookEvent($event);
			$duplicateEvent->save();
			$event->updateStatus(Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_DUPLICATED);
		}
	}

	/**
	 * Remove events older than given number of days
	 *
	 * @return void
	 */
	protected function _removeOldEvents()
	{		
		$eventCollection = Mage::getModel('koongoorderapi/webhook_event')->getCollection();
		$select = $eventCollection->getSelect();		
		$checkTime = $this->_helper()->getTime(self::EVENT_REMOVAL_CHECK_PERIOD);
		$select->where('updated_at < ?',$checkTime);
		$eventCollection->load();		
		foreach ($eventCollection as $event) {
			$event->delete();			
		}
	}

	/**
	 * Check processing events older than given preiod
	 * Renew the events into Pending state if possible.
	 * @return void
	 */
	protected function _renewProcessingEvents()
	{		
		$eventCollection = Mage::getModel('koongoorderapi/webhook_event')->getCollection();
		$select = $eventCollection->getSelect();
		$select->where('status = ?',Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_PROCESSING);
		$checkTime = $this->_helper()->getTime(self::EVENT_PROCESSING_CHECK_PERIOD);
		$select->where('updated_at < ?',$checkTime);
		$eventCollection->load();

		foreach ($eventCollection as $event) {
			
			$counter = $event->getDuplicityCounter();
			if($counter >= self::EVENT_RENEWAL_COUNTER_LIMIT)
			{
				$message = $this->_helper()->__("Event renewal limit (%s) reached.", self::EVENT_RENEWAL_COUNTER_LIMIT);
				$event->updateStatus(Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_ERROR, $message);
			}
			else
			{
				$counter++;
				$event->setDuplicityCounter($counter);
				$message = $this->_helper()->__("Event stacked at processing state. Event renewed.");
				$event->updateStatus(Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_PENDING, $message);
			}
		}
	}

	/**
	 * Process pending webhook events
	 * @param Array $eventIds Event ids that should be processed (process all events if null)
	 * @return void
	 */
	protected function _processPendingEvents($eventIds)
	{
		$eventCollection = Mage::getModel('koongoorderapi/webhook_event')->getCollection();
		$select = $eventCollection->getSelect();
		$select->where('status = ?',Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_PENDING);
		
		if(isset($eventIds) && !empty($eventIds))
		{
			$select->where('entity_id IN (?)',$eventIds);
		}
		$eventCollection->load();

		$webhooks = array();
		foreach ($eventCollection as $event) {
			$webhookId = $event->getWebhookId();	
			if(!isset($webhooks[$webhookId]))
			{
				$webhook = Mage::getModel('koongoorderapi/webhook')->load($webhookId);
				if($webhook->getId())
				{
					$webhooks[$webhookId] = $webhook;
				}
				else
				{ 
					$order = Mage::getModel('sales/order')->load($event->getOrderId());        
					if($order->getId())
					{
						//webhook is missing, try to add events to existing webhook
						$this->_getManager()->addWebhookEvents($event->getTopic(), $event->getOrderId(), $order->getStoreId(), $event->getParams());
					}
					$message = $this->_helper()->__("Missing webhook for event. Events for new webhooks recreated, if webhooks available");
					
					$event->setDuplicityCounter(self::EVENT_RENEWAL_COUNTER_LIMIT);
					$event->updateStatus(Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_ERROR, $message);
					continue;
				}
			}
			$this->_runEvent($event, $webhook);
		}
	}

	/**
	 * Run event webhooks
	 *
	 * @param Nostress_Koongoorderapi_Model_Webhook_Event $event
	 * @param Nostress_Koongoorderapi_Model_Webhook $webhook
	 * @return void
	 */
	protected function _runEvent($event, $webhook)
	{
		$client = $this->_getApiClient();
		try {
			$event->updateStatus(Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_PROCESSING);			
			$payload = $webhook->preparePayloadForEvent($event);												
			$response = $client->sendRequestPost($webhook->getUrl(), $payload, $webhook->getTopic(), $webhook->getSecret());						
			$event->updateStatus(Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_FINISHED,$response);			
		} catch (Exception $e) {
			$event->updateStatus(Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_ERROR, $e->getMessage());			
		}
	}

	/**
	 * Add cancel order webhook events
	 *
	 * @param Mage_Sales_Model_Order $order
	 * @param String $cancelReason
	 * @param boolean $invoiceExists
	 * @return Array Event Ids.
	 */
	public function addCancelOrderEvents($order, $cancelReason = "")
	{
		$topic = Nostress_Koongoorderapi_Model_Webhook::WEBHOOK_TOPIC_CANCEL_ORDER;
		return $this->_addWebhookEvents($topic, $order->getId(), $order->getStoreId(), array("note" => $cancelReason));				
	}
	
	/**
	 * Add shipment webhook events
	 *
	 * @param Mage_Sales_Model_Shipment $shipment
	 * @return Array Event Ids.
	 */
	public function addShipmentEvents($shipment)
	{
		$topic = Nostress_Koongoorderapi_Model_Webhook::WEBHOOK_TOPIC_ADD_SHIPMENT;
		$params = array("shipment_id" => $shipment->getId());
		return $this->_addWebhookEvents($topic, $shipment->getOrderId(), $shipment->getStoreId(), $params);				
	}

	/**
	 * Add webhook events to table
	 *
	 * @param String $topic Webhook topic
	 * @param Int $orderId
	 * @param Int $storeId
	 * @param Array $params
	 * @return Array Event Ids.
	 */
	protected function _addWebhookEvents($topic, $orderId, $storeId, $params)
	{		
		$webhooks = $this->_getWebhooks($storeId, $topic);
		
		$ids = array();
		foreach ($webhooks as $webhook) {
			$event = $this->_createWebhookEvent($webhook, $orderId, $params);			
			$event->save();
			$ids[] = $event->getId();
		}	
		return $ids;	
	}

	/**
	 * Get webhooks by storeId and topic
	 *
	 * @param Int $storeId
	 * @param String $topic
	 * @return void
	 */
	protected function _getWebhooks($storeId, $topic)
	{
		$webhooksCollection = Mage::getModel('koongoorderapi/webhook')->getCollection();
		$select = $webhooksCollection->getSelect();
		$select->where("store_id = ?", $storeId);
		$select->where("topic = ?", $topic);

		return $webhooksCollection->load();
	}

	/**
	 * Create event for webhook
	 *
	 * @param Nostress_Koongoorderapi_Model_Webhook $webhook
	 * @param Int $orderId
	 * @param Array $params
	 * @return void
	 */
	protected function _createWebhookEvent($webhook, $orderId, $params)
	{
		$event = Mage::getModel('koongoorderapi/webhook_event');
		$event->setTopic($webhook->getTopic());
		$event->setWebhookId($webhook->getId());
		$event->setOrderId($orderId);
		$event->setStatus(Nostress_Koongoorderapi_Model_Webhook_Event::STATUS_PENDING);
		$event->setParams(json_encode($params));
		$now = $this->_helper()->getNow();
		$event->setCreatedAt($now);
		$event->setupdatedAt($now);
		return $event;
	}

	/**
	 * Load API client
	 *
	 * @return Nostress_Koongoorderapi_Model_Client
	 */
	protected function _getApiClient()
	{
		if(!isset($this->_apiClient))
			$this->_apiClient = Mage::getModel('koongoorderapi/api_client_kaas');		
		return $this->_apiClient;
	}

	/**
	 * Get Manager for events
	 *
	 * @return Nostress_Koongoorderapi_Model_Webhook_Event_Manager
	 */
	protected function _getManager()
	{
		if(!isset($this->_eventManager))
			$this->_eventManager = Mage::getSingleton("koongoorderapi/webhook_event_manager");
		return $this->_eventManager;
	}

}
